import logging
from django.core.management.base import BaseCommand, CommandError
from django_pyres.conf import settings

from optparse import make_option
from pyres.horde import Khan
from pyres import setup_pidfile

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option("--pool", type="int", dest="pool_size", default=0, help="Number of minions to spawn under the manager."),
        make_option('-l', '--log-level', dest='log_level', default='info', help='log level.  Valid values are "debug", "info", "warning", "error", "critical", in decreasing order of verbosity. Defaults to "info" if parameter not specified.'),
        make_option('-f', dest='logfile', help='If present, a logfile will be used.  "stderr", "stdout", and "syslog" are all special values.'),
    )
    help = 'Creates and runs a pyres manager and worker pool.'

    def handle(self, queue_list, **options):
        queues = queue_list.split(',')
        log_level = getattr(logging, options['log_level'].upper(), 'INFO')
        pool_size = options.get('pool_size')
        log_file = options.get('logfile')
        if pool_size == 0:
            pool_size = settings.PYRES_MANAGER_POOL_SIZE
        setup_pidfile(settings.PYRES_WORKER_PIDFILE)
        worker = Khan(
            pool_size=pool_size,
            queues=queues,
            server=settings.PYRES_HOST,
            password=settings.PYRES_PASSWORD,
            logging_level=log_level,
            log_file=log_file
        )
        worker.work()
        #Khan.run(
        #    pool_size=pool_size, 
        #    queues=queues, 
        #    server=settings.PYRES_HOST, 
        #    logging_level=log_level, 
        #    log_file=None)



from django.conf import settings #noqa
from appconf import AppConf

class PyresConf(AppConf):
    HOST = "localhost:6379"
    PASSWORD = None
    USE_QUEUE=True
    WORKER_PIDFILE = None
    WORKER_INTERVAL = None
    WORKER_LOGFILE = None
    WORKER_TIMEOUT= None
    MANAGER_POOL_SIZE=2


    class Meta:
        prefix = 'pyres'



from functools import update_wrapper
from django_pyres.conf import settings
from django import db
from .core import pyres


def close_connection_after(func):
    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        if settings.PYRES_USE_QUEUE:
            db.close_connection()
        return result
    update_wrapper(wrapper, func)
    return wrapper

class Job(object):
    """
Class that wraps a function to enqueue in pyres
"""
    _resque_conn = pyres

    def __init__(self, func, queue):
        self.func = close_connection_after(func)
        #self.priority = priority

        # Allow this class to be called by pyres
        self.queue = str(queue)
        self.perform = self.func

        # Wrap func
        update_wrapper(self, func)

    # _resque wraps the underlying resque connection and delays initialization
    # until needed
    @property
    def _resque(self):
        return self._resque_conn

    @_resque.setter # NOQA
    def _resque(self, val):
        self._resque_conn = val

    def enqueue(self, *args, **kwargs):
        if settings.PYRES_USE_QUEUE:
            queue = kwargs.pop('queue', self.queue)
            if kwargs:
                raise Exception("Cannot pass kwargs to enqueued tasks")
            class_str = '%s.%s' % (self.__module__, self.__name__)
            self._resque.enqueue_from_string(class_str, queue, *args)
        else:
            return self.func(*args, **kwargs)

    def enqueue_at(self, dt, *args, **kwargs):
        queue = kwargs.pop('queue', self.queue)
        if kwargs:
            raise Exception('Cannot pass kwargs to enqueued tasks')
        class_str = '%s.%s' % (self.__module__, self.__name__)
        self._resque.enqueue_at_from_string(dt, class_str, queue, *args)

    def __call__(self, *args, **kwargs):
        return self.func(*args, **kwargs)

    def __repr__(self):
        return 'Job(func=%s, queue=%s)' % (self.func, self.queue)


def job(queue, cls=Job):
    def _task(f):
        return cls(f, queue)
    return _task

